﻿using System;
using System.Collections.Generic;

namespace ExtensibleCalculator
{
    // Bart De Smet: C# 5.0 Unleashed, page 816
    public class Calculator
    {
        private readonly IDictionary<string, Func<int, int, int>> _operations =
            new Dictionary<string, Func<int, int, int>>
            {
                {"add", (a, b) => a + b},
                {"sub", (a, b) => a - b},
                {"div", (a, b) => a / b}
            };

        public int Eval(int left, string operation, int right)
        {
            if (!_operations.ContainsKey(operation))
            {
                throw new ArgumentException("operation is illegal: " + operation);
            }
            Func<int, int, int> func = _operations[operation];
            return func(left, right);
        }

        // new functions can be added at run-time!!
        public void AddOperator(String name, Func<int, int, int> function)
        {
            _operations[name] = function;
        }
    }
}
